require("core-js/modules/es.array.filter.js");

require("core-js/modules/es.array.for-each.js");

require("core-js/modules/es.array.from.js");

require("core-js/modules/es.array.includes.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.array.map.js");

require("core-js/modules/es.array.sort.js");

require("core-js/modules/es.date.to-string.js");

require("core-js/modules/es.function.name.js");

require("core-js/modules/es.object.define-property.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.regexp.to-string.js");

require("core-js/modules/es.set.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/esnext.set.add-all.js");

require("core-js/modules/esnext.set.delete-all.js");

require("core-js/modules/esnext.set.difference.js");

require("core-js/modules/esnext.set.every.js");

require("core-js/modules/esnext.set.filter.js");

require("core-js/modules/esnext.set.find.js");

require("core-js/modules/esnext.set.intersection.js");

require("core-js/modules/esnext.set.is-disjoint-from.js");

require("core-js/modules/esnext.set.is-subset-of.js");

require("core-js/modules/esnext.set.is-superset-of.js");

require("core-js/modules/esnext.set.join.js");

require("core-js/modules/esnext.set.map.js");

require("core-js/modules/esnext.set.reduce.js");

require("core-js/modules/esnext.set.some.js");

require("core-js/modules/esnext.set.symmetric-difference.js");

require("core-js/modules/esnext.set.union.js");

require("core-js/modules/web.dom-collections.for-each.js");

require("core-js/modules/web.dom-collections.iterator.js");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/* eslint-disable no-underscore-dangle */
var yaml = require('js-yaml');

var _require = require('../helpers/evaluate_module_from_source'),
    evaluateModuleFromSource = _require.evaluateModuleFromSource;

var PLUGIN_NAME = 'GraphqlKnownOperationsPlugin';
var GRAPHQL_PATH_REGEX = /(query|mutation)\.graphql$/;
/**
 * Returns whether a given webpack module is a "graphql" module
 */

var isGraphqlModule = function isGraphqlModule(module) {
  return GRAPHQL_PATH_REGEX.test(module.resource);
};
/**
 * Returns graphql operation names we can parse from the given module
 *
 * Since webpack gives us the source **after** the graphql-tag/loader runs,
 * we can look for specific lines we're guaranteed to have from the
 * graphql-tag/loader.
 */


var getOperationNames = function getOperationNames(module) {
  var originalSource = module.originalSource();

  if (!originalSource) {
    return [];
  }

  var _evaluateModuleFromSo = evaluateModuleFromSource(originalSource.source().toString(), {
    // what: stub require(...) when evaluating the graphql module
    // why: require(...) is used to fetch fragments. We only need operation metadata, so it's fine to stub these out.
    require: function require() {
      return {
        definitions: []
      };
    }
  }),
      moduleExports = _evaluateModuleFromSo.exports;

  var names = moduleExports.definitions.filter(function (x) {
    return ['query', 'mutation'].includes(x.operation);
  }).map(function (x) {
    var _x$name;

    return (_x$name = x.name) === null || _x$name === void 0 ? void 0 : _x$name.value;
  }) // why: It's possible for operations to not have a name. That violates our eslint rule, but either way, let's ignore those here.
  .filter(Boolean);
  return names;
};

var createFileContents = function createFileContents(knownOperations) {
  var sourceData = Array.from(knownOperations.values()).sort(function (a, b) {
    return a.localeCompare(b);
  });
  return yaml.dump(sourceData);
};
/**
 * Creates a webpack4 compatible "RawSource"
 *
 * Inspired from https://sourcegraph.com/github.com/FormidableLabs/webpack-stats-plugin@e050ff8c362d5ddd45c66ade724d4a397ace3e5c/-/blob/lib/stats-writer-plugin.js?L144
 */


var createWebpackRawSource = function createWebpackRawSource(source) {
  var buff = Buffer.from(source, 'utf-8');
  return {
    source: function source() {
      return buff;
    },
    size: function size() {
      return buff.length;
    }
  };
};

var onSucceedModule = function onSucceedModule(_ref) {
  var module = _ref.module,
      knownOperations = _ref.knownOperations;

  if (!isGraphqlModule(module)) {
    return;
  }

  getOperationNames(module).forEach(function (name) {
    return knownOperations.add(name);
  });
};

var onCompilerEmit = function onCompilerEmit(_ref2) {
  var compilation = _ref2.compilation,
      knownOperations = _ref2.knownOperations,
      filename = _ref2.filename;
  var contents = createFileContents(knownOperations);
  var source = createWebpackRawSource(contents);
  var asset = compilation.getAsset(filename);

  if (asset) {
    compilation.updateAsset(filename, source);
  } else {
    compilation.emitAsset(filename, source);
  }
};
/**
 * Webpack plugin that outputs a file containing known graphql operations.
 *
 * A lot of the mechanices was expired from [this example][1].
 *
 * [1]: https://sourcegraph.com/github.com/FormidableLabs/webpack-stats-plugin@e050ff8c362d5ddd45c66ade724d4a397ace3e5c/-/blob/lib/stats-writer-plugin.js?L136
 */


var GraphqlKnownOperationsPlugin = /*#__PURE__*/function () {
  "use strict";

  function GraphqlKnownOperationsPlugin(_ref3) {
    var filename = _ref3.filename;

    _classCallCheck(this, GraphqlKnownOperationsPlugin);

    this._filename = filename;
  }

  _createClass(GraphqlKnownOperationsPlugin, [{
    key: "apply",
    value: function apply(compiler) {
      var _this = this;

      var knownOperations = new Set();
      compiler.hooks.emit.tap(PLUGIN_NAME, function (compilation) {
        onCompilerEmit({
          compilation: compilation,
          knownOperations: knownOperations,
          filename: _this._filename
        });
      });
      compiler.hooks.compilation.tap(PLUGIN_NAME, function (compilation) {
        compilation.hooks.succeedModule.tap(PLUGIN_NAME, function (module) {
          onSucceedModule({
            module: module,
            knownOperations: knownOperations
          });
        });
      });
    }
  }]);

  return GraphqlKnownOperationsPlugin;
}();

module.exports = GraphqlKnownOperationsPlugin;
